function [b_core, g_core, h_core, sg_core, sh_core, s_lnqCL, lnpCL] ...
                    = VARCL_rs( Y_all, X_all, W_all, beta0, Vbeta, ...
                    simresults, wCL, coreY, coreX, otherX, coreW, ...
                    kgam, Tt, n, ncore, nsims, doparallel, s_lnqCL )
%% Resample draws from the composite posterior using rejection sampling

maxtries = 200; % maximum number of loops before giving up

m_all = length( simresults );
svsims = size( simresults( 1 ).beta, 1 );

if ~isempty( s_lnqCL )
    qCL = exp( s_lnqCL - max( s_lnqCL( : ) ) );
    lnsumqCL = log( sum( bsxfun( @times, qCL, wCL' ), 2 ) ) + max( s_lnqCL( : ) );
    lnpCL = sum( bsxfun( @times, s_lnqCL, wCL' ), 2 );
else
    s_lnqCL = nan( m_all * svsims, m_all );
    lnsumqCL = nan( m_all * svsims, 1 );
    lnpCL = nan( m_all * svsims, 1 );
end

a_mi = nan( nsims, 1 );
a_isim = nan( nsims, 1 );
naccepted = 0;
accept_rate = 1;
count = 0;

while naccepted < nsims && count <= maxtries
    % get a set of proposals
    npropose = max( min( floor( ( nsims - naccepted ) / accept_rate ), 10 * ( nsims - naccepted ) ), 100 );
    mi_draws = 1 + sum( bsxfun( @gt, rand( npropose, 1 ), cumsum( wCL' ) ), 2 );
    isim_draws = ceil( svsims * rand( npropose, 1 ) );
    mi_isim = ( isim_draws - 1 ) * m_all + mi_draws;
    
    new = isnan( lnpCL( mi_isim ) );
    if nnz( new ) > 0
        % evaluate the sub-model log-likelihoods for new draws
        lnqCL = eval_CL_components( Y_all, X_all, W_all, beta0, Vbeta, ...
                simresults, coreY, coreX, otherX, coreW, kgam, Tt, n, ncore, ...
                mi_draws( new ), isim_draws( new ), doparallel );

        % construct the proposal and target from log-likelihood evaluations
        qCL = exp( lnqCL - max( lnqCL( : ) ) );
        lnsumqCL( mi_isim( new ) ) = log( sum( bsxfun( @times, qCL, wCL' ), 2 ) ) + max( lnqCL( : ) );
        lnpCL( mi_isim( new ) ) = sum( bsxfun( @times, lnqCL, wCL' ), 2 );
        s_lnqCL( mi_isim( new ), : ) = lnqCL;
    end

    % sample and update
    accept = log( rand( npropose, 1 ) ) < lnpCL( mi_isim ) - lnsumqCL( mi_isim );
    accept( cumsum( accept ) > nsims - naccepted ) = false;
    
    a_mi( naccepted + 1:naccepted + nnz( accept ) ) = mi_draws( accept );
    a_isim( naccepted + 1:naccepted + nnz( accept ) ) = isim_draws( accept );
    naccepted = naccepted + nnz( accept );
    accept_rate = nnz( accept ) / npropose;
    
    count = count + 1;
end

% if failed to accept all required draws, resample the accepted ones
if naccepted < nsims
%     nnz( isnan( a_mi( 1:naccepted ) ) )
    if naccepted > 0
%         disp( ['resampling from ' num2str( naccepted ) ' valid draws.'] );
        resamp = ceil( naccepted * rand( nsims - naccepted, 1 ) );
        a_mi( naccepted + 1:end ) = a_mi( resamp );
        a_isim( naccepted + 1:end ) = a_isim( resamp );
    else
        % we're in trouble
        error( 'VAR-CL Rejection Sampling: failed to get a single valid draw.' );
    end
%     invalid_mi = nnz( isnan( a_mi ) ), invalid_isim = nnz( isnan( a_isim ) )
end

% save draws according to the resampled indices
b_core = zeros( nsims, nnz( coreX ) );
g_core = zeros( nsims, nnz( coreW ) );
h_core = zeros( nsims, nnz( coreY ) );
sg_core = zeros( nsims, nnz( coreW( 1:kgam ) ) );
sh_core = zeros( nsims, ncore );

for j = 1:nsims
    b_core( j, : ) = simresults( a_mi( j ) ).beta( a_isim( j ), coreX );
    g_core( j, : ) = simresults( a_mi( j ) ).gam( a_isim( j ), coreW );
    h_core( j, : ) = simresults( a_mi( j ) ).h( a_isim( j ), coreY );
    sg_core( j, : ) = simresults( a_mi( j ) ).Siggam( a_isim( j ), coreW( 1:kgam ) );
    sh_core( j, : ) = simresults( a_mi( j ) ).Sigh( a_isim( j ), 1:ncore );
end

%     if count < maxtries
%         % propose a subset of draws
%         npropose = max( min( floor( ( nsims - naccepted ) / accept_rate ), 10 * ( nsims - naccepted ) ), 100 );
%         mi_draws = 1 + sum( bsxfun( @gt, rand( npropose, 1 ), cumsum( wCL' ) ), 2 );
%         isim_draws = ceil( svsims * rand( npropose, 1 ) );
%         mi_isim = ( isim_draws - 1 ) * m_all + mi_draws;
%     else
%         % evaluate all the draws
%         npropose = m_all * svsims;
%         mi_draws = repmat( 1:m_all, 1, svsims )';
%         isim_draws = reshape( repmat( 1:svsims, m_all, 1 ), m_all * svsims, [] );
%         mi_isim = ( isim_draws - 1 ) * m_all + mi_draws;
%     end
