function s_fcst = VARSV_fcst( Y, X, s_beta, s_gam, s_h, s_Siggam, s_Sigh, ncore, kgam, nsims, loff )
%% generate point, density and CPRS forecast evals

[ttmax, n] = size( Y );
if loff > 1
    % assume we're doing direct forecasting only
    ttmax = 1;
end

% preconstruct useful indices
icov = triu( repmat( 2:n, n - 1, 1 ) );
icov = reshape( icov, [], 1 );
icov( icov == 0 ) = [];
jcov = triu( repmat( ( 1: n - 1 )', 1, n - 1 ) );
jcov = reshape( jcov, [], 1 );
jcov( jcov == 0 ) = [];

% allocate space for storage
s_fcst = nan( nsims, ttmax, ncore + 1, 5 );

% process all saved draws
for isim = 1:nsims
    % initialize data
    fX1 = X( 1, : ); fX2 = fX1;
    
    fbeta1 = s_beta( isim, : )'; fbeta2 = fbeta1;
    fgam1 = s_gam( isim, end - kgam + 1:end )'; fgam2 = fgam1;
    fh1 = s_h( isim, end - n + 1:end )'; fh2 = fh1;
    fSigh1 = s_Sigh( isim, : )'; fSigh2 = fSigh1;
    fSiggam1 = s_Siggam( isim, : )'; fSiggam2 = fSiggam1;

    for tt = 1:ttmax
        % generate fcst of gam_t and h_t
        for count = 1:loff
            fgam1 = fgam1 + sqrt( fSiggam1 ) .* randn( kgam, 1 );
            fh1 = fh1 + sqrt( fSigh1 ) .* randn( n, 1 );
        end

        % generate fcst of y_t
        B = speye( n ) + sparse( icov, jcov, fgam1, n, n );
        ifSig1 = B' * sparse( 1:n, 1:n, exp( -fh1 ) ) * B;
        ym1 = B \ ( kron( speye( n ), fX1 ) * fbeta1 );
        yhat_1 = ym1 + chol( ifSig1 ) \ randn( n, 1 );

        % update
        fX1 = [1 yhat_1' fX1( 2:end - n ) ];
        yhat_c1 = yhat_1( 1:ncore );

        %% repeat for CPRS
        % generate fcst of gam_t and h_t
        for count = 1:loff
            fgam2 = fgam2 + sqrt( fSiggam2 ) .* randn( kgam, 1 );
            fh2 = fh2 + sqrt( fSigh2 ) .* randn( n,1 );
        end

        % generate fcst of y_t
        B = speye( n ) + sparse( icov, jcov, fgam2, n, n );
        ifSig2 = B' * sparse( 1:n, 1:n, exp( -fh2 ) ) * B;
        ym2 = B \ ( kron( speye( n ), fX2 ) * fbeta2 );
        yhat_2 = ym2 + chol( ifSig2 ) \ randn( n, 1 );

        % update
        fX2 = [1 yhat_2' fX2( 2:end - n )];
        yhat_c2 = yhat_2( 1:ncore );

        % Compute the precision for ycore
        ifSig1_c = ifSig1( 1:ncore, 1:ncore ) - ( ifSig1( 1:ncore, ncore + 1:end ) ...
                / ifSig1( ncore + 1:end, ncore + 1:end ) ) ...
                * ifSig1( 1:ncore, ncore + 1:end )';
        CifSig1_c = chol( ifSig1_c );
        lndet_ifSig1_c = 2 * sum( log( diag( CifSig1_c ) ) );

        % compute the marginal variances for ycore
        if ncore == 3
            dfSig1_c = [ifSig1_c( 2, 2 ) * ifSig1_c( 3, 3 ) - ifSig1_c( 2, 3 ) * ifSig1_c( 3, 2 ); ...
                        ifSig1_c( 1, 1 ) * ifSig1_c( 3, 3 ) - ifSig1_c( 1, 3 ) * ifSig1_c( 3, 1 ); ...
                        ifSig1_c( 1, 1 ) * ifSig1_c( 2, 2 ) - ifSig1_c( 1, 2 ) * ifSig1_c( 2, 1 )] ...
                        * exp( -lndet_ifSig1_c );
        else
            dfSig1_c = diag( inv( ifSig1_c ) );
        end
        
        % generate forecast statistics
        s_fcst( isim, tt, :, : ) = [[yhat_c1; nan] [yhat_c2; nan] ...
            [( yhat_c1 - Y( tt, 1:ncore )' ) .^ 2; nan] ...
            [abs( yhat_c1 - Y( tt, 1:ncore )' ) - .5 * abs( yhat_c1 - yhat_c2 ); nan] ...
            exp( [-0.5 * ( log( dfSig1_c ) + ( Y( tt, 1:ncore )' - ym1( 1:ncore ) ) .^ 2 ./ dfSig1_c ); ...
            0.5 * ( lndet_ifSig1_c - sum( ( CifSig1_c * ( Y( tt, 1:ncore )' - ym1( 1:ncore ) ) ) .^ 2 ) )] )];
    end
end