function [beta0, Vbeta, agam, Vgam, ah, Vh, nugam0, Sgam0, nuh0, Sh0, prior_all, prior_core, prior_other] = ...
    construct_priors( Y_all, X_all, coreY, otherX, coreW, n, T, p, kbeta, kgam, m_all )
%% construct the priors

% construct shrinkage priors by averaging Minnesota priors over all models
beta0 = 0; Vbeta = 0;
for mi = 1:m_all
    [beta0_i, Vbeta_i] = construct_shrinkageprior( Y_all{mi}( 1:T, : ), X_all{mi}( 1:T, : ), kbeta, n, T, p );
    beta0 = beta0 + beta0_i / m_all;
    Vbeta = Vbeta + Vbeta_i / m_all;
end

agam = zeros( kgam, 1 ); Vgam = 10 * ones( kgam, 1 );
ah = zeros( n, 1 ); Vh = 10 * ones( n, 1 );
nugam0 = 10 * ones( kgam, 1 ); Sgam0 = .01 ^ 2 * ones( kgam, 1 ) .* ( nugam0 - 1 ); 
nuh0 = 10 * ones( n, 1 ); Sh0 = .1 ^ 2 * ones( n, 1 ) .* ( nuh0 - 1 );

%% inline function to evaluate the prior
cpri_all = -.5 * kbeta * log( 2 * pi ) - .5 * sum( log( Vbeta ) ) ...
    + nugam0' * log( Sgam0 ) - sum( gammaln( nugam0 ) ) ...
    + nuh0' * log( Sh0 ) - sum( gammaln( nuh0 ) ) ...
    - .5 * ( kgam + n ) * log( 2 * pi ) ...
    - .5 * sum( log( Vgam ) ) - .5 * sum( log( Vh ) );

prior_all = @( b, sg, sh, a0, b0 ) cpri_all ...
    - .5 * ( ( b - beta0 ) ./ Vbeta )' * ( b - beta0 ) ...
    - ( nugam0 + 1 )' * log( sg ) - sum( Sgam0 ./ sg ) ... 
    - ( nuh0 + 1 )' * log( sh ) - sum( Sh0 ./ sh ) ...
    - .5 * ( ( a0 - agam ) ./ Vgam )' * ( a0 - agam ) ...
    - .5 * ( ( b0 - ah ) ./ Vh )' * ( b0 - ah );

%% inline function to evaluate the prior of the 'Ycore' VARX-SV subsystem
cpri_core = -.5 * nnz( ~otherX ) * log( 2 * pi ) - .5 * sum( log( Vbeta( ~otherX ) ) ) ...
    + nugam0( coreW( 1:kgam ) )' * log( Sgam0( coreW( 1:kgam ) ) ) ...
    - sum( gammaln( nugam0( coreW( 1:kgam ) ) ) ) ...
    + nuh0( coreY( 1:n ) )' * log( Sh0( coreY( 1:n ) ) ) ...
    - sum( gammaln( nuh0( coreY( 1:n ) ) ) ) ...
    - .5 * (  nnz( coreW( 1:kgam ) ) + nnz( coreY( 1:n ) ) ) * log( 2 * pi ) ...
    - .5 * sum( log( Vgam( coreW( 1:kgam ) ) ) ) - .5 * sum( log( Vh( coreY( 1:n ) ) ) );

prior_core = @( b, sg, sh, a0, b0 ) cpri_core ...
    - .5 * sum( ( b - beta0( ~otherX ) ) .^ 2 ./ Vbeta( ~otherX ) ) ...
    - ( nugam0( coreW( 1:kgam ) ) + 1 )' * log( sg ) - sum( Sgam0( coreW( 1:kgam ) ) ./ sg ) ...
    - ( nuh0( coreY( 1:n ) ) + 1 )' * log( sh ) - sum( Sh0( coreY( 1:n ) ) ./ sh ) ...
    - .5 * sum( ( a0 - agam( coreW( 1:kgam ) ) ) .^ 2 ./ Vgam( coreW( 1:kgam ) ) ) ...
    - .5 * sum( ( b0 - ah( coreY( 1:n ) ) ) .^ 2 ./ Vh( coreY( 1:n ) ) );

%% inline function to evaluate the prior of the 'Yother' equation parameters
cpri_other = -.5 * nnz( otherX ) * log( 2 * pi ) - .5 * sum( log( Vbeta( otherX ) ) ) ...
    + nugam0( ~coreW( 1:kgam ) )' * log( Sgam0( ~coreW( 1:kgam ) ) ) ...
    - sum( gammaln( nugam0( ~coreW( 1:kgam ) ) ) ) ...
    + nuh0( ~coreY( 1:n ) )' * log( Sh0( ~coreY( 1:n ) ) ) ...
    - sum( gammaln( nuh0( ~coreY( 1:n ) ) ) ) ...
    - .5 * (  nnz( ~coreW( 1:kgam ) ) + nnz( ~coreY( 1:n ) ) ) * log( 2 * pi ) ...
    - .5 * sum( log( Vgam( ~coreW( 1:kgam ) ) ) ) - .5 * sum( log( Vh( ~coreY( 1:n ) ) ) );

prior_other = @( b, sg, sh, a0, b0 ) cpri_other ...
    - .5 * sum( ( b - beta0( otherX ) ) .^ 2 ./ Vbeta( otherX ) ) ...
    - ( nugam0( ~coreW( 1:kgam ) ) + 1 )' * log( sg ) - sum( Sgam0( ~coreW( 1:kgam ) ) ./ sg ) ...
    - ( nuh0( ~coreY( 1:n ) ) + 1 )' * log( sh ) - sum( Sh0( ~coreY( 1:n ) ) ./ sh ) ...
    - .5 * sum( ( a0 - agam( ~coreW( 1:kgam ) ) ) .^ 2 ./ Vgam( ~coreW( 1:kgam ) ) ) ...
    - .5 * sum( ( b0 - ah( ~coreY( 1:n ) ) ) .^ 2 / Vh( ~coreY( 1:n ) ) );