function lnqCL = eval_CL_components( Y_all, X_all, W_all, beta0, Vbeta, ...
                 simresults, coreY, coreX, otherX, coreW, ...
                 kgam, T, n, ncore, mi_draws, isim_draws, doparallel, showprog )

[~, u1idx, u2idx] = unique( [mi_draws isim_draws], 'rows' );
umi_draws = mi_draws( u1idx );
uisim_draws = isim_draws( u1idx );

nsims = length( u1idx );
m_all = length( simresults );
[svsims, kbeta] = size( simresults( 1 ).beta );

if nargin < 18
    showprog = nsims >= 100;
end

if nargin < 17
    doparallel = true;
end

if showprog
    [~, ppm] = setup_progmons( doparallel, ['Evaluating likelihoods for ' num2str( nsims ) ' draws: '], nsims );
else
    ppm = [];
end

beta0 = beta0( ~coreX & ~otherX );
Vbeta = Vbeta( ~coreX & ~otherX );
iVbeta = diag( 1 ./ Vbeta );

%% prepare stored draws for parfor loop
% first, create seperate variables for each set of draws
store_beta = reshape( permute( reshape( [simresults.beta], ...
    [svsims kbeta m_all] ), [3 1 2] ), m_all * svsims, kbeta );
store_gam = reshape( permute( reshape( [simresults.gam], ...
    [svsims kgam * T m_all] ), [3 1 2] ), m_all * svsims, kgam * T );
store_h = reshape( permute( reshape( [simresults.h], ...
    [svsims n * T m_all] ), [3 1 2] ), m_all * svsims, n * T );
mlx = reshape( [simresults.mlx], 2, m_all );

% now, "slice" the draws by parfor loop iterations
umi_isim = ( uisim_draws - 1 ) * m_all + umi_draws;
store_beta = store_beta( umi_isim, coreX );
store_gam = store_gam( umi_isim, coreW );
store_h = store_h( umi_isim, coreY );
mlx = mlx( 1, : );

ulnqCL = nan( nsims, m_all );
parfor jsim = 1:nsims
    beta = store_beta( jsim, : )';
    gam = store_gam( jsim, : )';
    h = store_h( jsim, : )';
    
    for mj = 1:m_all
        y = reshape( Y_all{mj}( 1:T, : )', [], 1 ); y = y( coreY );
        X = SURform2( X_all{mj}( 1:T, : ), n );     X1 = X( coreY, coreX ); X2 = X( coreY, ~coreX & ~otherX );
        W = W_all{mj}( 1:T * n, 1:T * kgam );       W = W( coreY, coreW );

        u = y - X1 * beta - X2* beta0 - W * gam;
        bigiSig = spdiags( exp( -h ), 0, ncore * T, ncore * T );
        X2bigiSig = X2' * bigiSig;
        X2bigiSigu= X2bigiSig * u;

        Kbeta = iVbeta + X2bigiSig * X2;
        CKbeta = chol( Kbeta, 'lower' );
        z = CKbeta \ X2bigiSigu;
        
        llike = - sum( log( diag( CKbeta ) ) ) - 0.5 * ( ...
            sum( h ) + sum( log( Vbeta ) ) + u' * ( exp( -h ) .* u ) - z' * z );

        ulnqCL( jsim, mj ) = llike - mlx( mj );
    end

    % update the progress monitor
    if ~isempty( ppm )
        ppm.increment();
    end
end

% close the progress window if it exists
if ~isempty( ppm )
    ppm.delete();
end

lnqCL = ulnqCL( u2idx, : );